--
-- Database schema for IR databases
--
-- This is an sqlite schema for storing and classifying IR symbols
-- (as defined by the ir package). Symbols are stored as Python
-- ir.Symbol objects, and are matched using ir.Symbol match operations.
-- The database keeps semantic information about Symbols, such as type
-- of operation, features, and icon images.
-- This is for the use of higher-level interfaces such as UI generators
-- and does not change the effect the Symbol has when emitted, though it may
-- help software emit it more intelligently.
--
-- Copyright (C)2010-2011 Perry The Cynic. All Rights Reserved.
--
PRAGMA foreign_keys = ON;


--
-- The config table has exactly one record.
-- It contains global configuration information.
--
CREATE TABLE config (
	version		INTEGER NOT NULL,			-- file format version
	creator		TEXT NULL,					-- creator name
	description	TEXT NULL					-- database description
);
insert into config (version) values (3);


--
-- Detailed information about symbol groups
--
CREATE TABLE symbolgroup (
	name		TEXT PRIMARY KEY,			-- group name
	description	TEXT NULL,					-- more detailed description
	organization TEXT NULL					-- associated company or organization
);


--
-- Individual Symbols
--
CREATE TABLE symbol (
	--
	-- Symbols are indexed by symbol grouping and name
	--
	grouping	TEXT NOT NULL REFERENCES symbolgroup(name),
	name		TEXT NOT NULL,

	--
	-- Essential columns: Must be present to store and find Symbols.
	--
	symbol		PICKLED NOT NULL,			-- whole Symbol, pickled
	type		TEXT NOT NULL,				-- symbol (encoding) type
	carrier		INTEGER NOT NULL			-- carrier frequency (marginally useful)
				CHECK (carrier > 20000 and carrier < 100000),

	--
	-- Operational characteristics
	--
	postdelay	FLOAT NULL,					-- seconds needed for device to settle

	--
	-- Semantic characteristics
	--
	category	TEXT NULL,					-- operation category
	features	TEXT NULL,					-- list of features (comma separated)
	opposite	TEXT NULL,					-- inverse symbol if any
	icon		TEXT NULL,					-- icon file or key

	--
	-- Miscellanea
	--
	description	TEXT NULL,					-- human-readable description
	notes		TEXT NULL,					-- note and oddities (not for user consumption)

	--
	-- Table constraints
	--
	CONSTRAINT master_key PRIMARY KEY ( grouping, name )
);
