#
# Global Cache interface plugin for Indigo 5
#
# Copyright 2011-2016 Perry The Cynic. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#	http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import os
import cyin
import cyin.asynplugin
import ir.db
import ir.context
from cyin import iom, plug
from cyin import debug, error
from cyin.asynplugin import asyncmethod
from cyin.check import *

import netdev
import irdev


#
# Menu generators
#
def _dev_menu(devclass):
	menu = [("%d@%s" % (dev.id, addr), "%s @ %s" % (dev.name, addr))
		for (dev, addr) in netdev.GCNet.all_devices(devclass)]
	return sorted(menu, key=lambda s: s[1])

class ReceiverFilter(cyin.MenuFilter):
	def evaluate(self):
		menu = _dev_menu(ir.gcache.Serial)
		for path, name in indigo.server.getSerialPorts(filter="indigo.ignoreBluetooth").items():
			menu.append((path, name))
		return menu

class EmitterFilter(cyin.MenuFilter):
	def evaluate(self):
		return _dev_menu(ir.gcache.IREmitter)

class RelayFilter(cyin.MenuFilter):
	def evaluate(self):
		return _dev_menu(ir.gcache.Relay)

class SensorFilter(cyin.MenuFilter):
	def evaluate(self):
		return _dev_menu(ir.gcache.Sensor)

class LearnerFilter(cyin.MenuFilter):
	def evaluate(self):
		return (
			[(dev.id, dev.name) for dev in irdev.IRReceiver.all_attr(lambda dev: dev.ready())]
			+
			[(dev.id, dev.name) for dev in netdev.GCNet.all_attr(lambda dev: dev.ready() and dev.gcnet.can_learn)]
			)


#
# The plugin
#
class Plugin(cyin.asynplugin.Plugin, ir.context.Context):

	irdb_path = cyin.PluginPreference(name="database", required=False, eval=True,
		check=[check_path, check_makes(ir.db.IRDb, "invalid IR database")])
	learner = cyin.PluginPreference(type=cyin.device, required=False, eval=True)

	def __init__(self, *args):
		cyin.asynplugin.Plugin.__init__(self, *args)
		ir.context.Context.__init__(self)

	def startup(self):
		cyin.asynplugin.Plugin.startup(self)
		self.schedule(lambda ctx: self.dbconfig())


	#
	# Configure use of an optional IR symbol database
	#
	@asyncmethod
	def dbconfig(self):
		if self.irdb_path:
			self.set_db(ir.db.IRDb(self.irdb_path))
			debug("using IR database", self.irdb_path)
		else:
			self.set_db(None)
			debug("using no IR database")


	#
	# ConfigUI for plugin preferences
	#
	class UI(cyin.ConfigUI):
		def end_ui(self, success):
			if success:
				cyin.plugin.dbconfig()
